// Copyright 2016 Cong Ding
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package stun

import (
	"net"
)

func (c *Client) test1(conn net.PacketConn, addr net.Addr) (*response, error) {
	return c.sendBindingReq(conn, addr, false, false)
}

func (c *Client) test2(conn net.PacketConn, addr net.Addr) (*response, error) {
	return c.sendBindingReq(conn, addr, true, true)
}

func (c *Client) test3(conn net.PacketConn, addr net.Addr) (*response, error) {
	return c.sendBindingReq(conn, addr, false, true)
}
